function [ x y Ex Ey ] = read_zbf( filename )
%READ_ZBF Reads a zemax beam file (ZBF)
%
% INPUT   -  filename: string filename of the ZBF file to be read
%
% OUTPUTS -  x,y: matrices with x and y coordinates of pixel centers
%                (in ZEMAX lens units
%            Ex,Ey: Complex E-field amplitudes in the ZBF file
%                   Ey = 0 is returned if the ZBF file is unpolarized

% read the binary data from the ZBF file
[ is_polarized nx ny dx dy cax cay ] = read_zbf_mex(filename);

% convert cax and cay arrays into complex Matlab matrices
x = zeros(nx,ny);
y = zeros(nx,ny);
Ex_real = zeros(nx,ny);
Ex_imag = zeros(nx,ny);
if is_polarized
    Ey_real = zeros(nx,ny);
    Ey_imag = zeros(nx,ny);
end
xc = 1 + nx/2;
yc = 1 + ny/2;
k = 1;
for j=1:ny
    for i=1:nx
        % populate x and y matrices
        x(i,j) = (i - xc)*dx;
        y(i,j) = (j - yc)*dy;
        
        % populate E-field matrices
        Ex_real(i,j) = cax(k);
        Ex_imag(i,j) = cax(k+1);
        if is_polarized
            Ey_real(i,j) = cay(k);
            Ey_imag(i,j) = cay(k+1);
        end
        
        % increment counter
        k = k + 2;
    end
end

% shift x and y arrays by half a pixel to make the pcolor plots come out right
x = x - dx/2;
y = y - dy/2;

% combine the real and imaginary parts at the end for speed
Ex = Ex_real + 1i.*Ex_imag;
if is_polarized
    Ey = Ey_real + 1i.*Ey_imag;
else
    Ey = 0;
end

end

