#include "mex.h"

FILE* get_housekeeping_data(char*, double*,
                            double*, double*, double*, double*);

void get_beam_maps(FILE*, double*, double*,
                   double*, double*, double*);

FILE* get_housekeeping_data(char* filename, double* is_polarized,
                            double* nx, double* ny, double* dx, double* dy) {
    mexPrintf("Opening beam file '%s\' for reading...\n", filename);
    mexEvalString("drawnow;");
    
    FILE* out = fopen(filename, "rb");
    
    // read the format version number
    int i = 0;
    fread(&i, sizeof(int), 1, out);
    // read the beam nx size
    int tmp_nx,tmp_ny;
    fread(&tmp_nx, sizeof(int), 1, out);
    nx[0] = (double) tmp_nx;
    // read the beam ny size
    fread(&tmp_ny, sizeof(int), 1, out);
    ny[0] = (double) tmp_ny;
    
    // read the ispolarized flag
    int tmp_is_pol = 0;
    fread(&tmp_is_pol, sizeof(int), 1, out);
    is_polarized[0] = (double) tmp_is_pol;
    
    // read out the current units NOT USED
    int unittype = 0;
    fread(&unittype, sizeof(int), 1, out);
    // 4 unused integers
    i = 0;
    fread(&i, sizeof(int), 1, out);
    fread(&i, sizeof(int), 1, out);
    fread(&i, sizeof(int), 1, out);
    fread(&i, sizeof(int), 1, out);
    
    // read dx
    double tmp_dx;
    fread(&tmp_dx, sizeof(double), 1, out);
    dx[0] = tmp_dx;
    // read dy
    double tmp_dy;
    fread(&tmp_dy, sizeof(double), 1, out);
    dy[0] = tmp_dy;
    
    // read beam parameters NOT USED
    double position, rayleigh, lambda, waist;
    fread(&position, sizeof(double), 1, out);
    fread(&rayleigh, sizeof(double), 1, out);
    fread(&lambda,   sizeof(double), 1, out);
    fread(&waist,    sizeof(double), 1, out);
    
    // fiber coupling NOT USED
    double ficl;
    fread(&ficl, sizeof(double), 1, out);
    // 4 unused doubles NOT USED
    double x;
    fread(&x, sizeof(double), 1, out);
    fread(&x, sizeof(double), 1, out);
    fread(&x, sizeof(double), 1, out);
    fread(&x, sizeof(double), 1, out);
    
    return out;
}

void get_beam_maps(FILE* out, double* cax, double* cay,
                   double* is_polarized, double* nx, double* ny) {
    // now the beam itself
    fread(&cax[0], sizeof(double), ((int) nx[0])*((int) ny[0])*2, out);
    if ((int) is_polarized[0]) {
        fread(&cay[0], sizeof(double), ((int) nx[0])*((int) ny[0])*2, out);
    } else {
        cay[0] = 0;
    }
    fclose(out);
    
    mexPrintf("Finished reading ZBF file\n");
    mexEvalString("drawnow;");
}

// gateway function
void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {
    // get input filename
    int buflen = mxGetNumberOfElements(prhs[0]) + 1;
    char* filename = mxCalloc(buflen, sizeof(char));
    
    // copy the string data
    if (mxGetString(prhs[0], filename, buflen) != 0)
        mexErrMsgTxt("Could not convert string data.");
    
    // declare output variables except for the beam maps
    double *is_polarized;
    double *nx;
    double *ny;
    double *dx;
    double *dy;
    
    // create the output variables except for the beam maps
    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
    plhs[2] = mxCreateDoubleMatrix(1, 1, mxREAL);
    plhs[3] = mxCreateDoubleMatrix(1, 1, mxREAL);
    plhs[4] = mxCreateDoubleMatrix(1, 1, mxREAL);
    
    // get a pointer to the output variables except for the beam maps
    is_polarized = mxGetPr(plhs[0]);
    nx = mxGetPr(plhs[1]);
    ny = mxGetPr(plhs[2]);
    dx = mxGetPr(plhs[3]);
    dy = mxGetPr(plhs[4]);
    
    // get everything but the beam maps
    FILE* out = get_housekeeping_data(filename, is_polarized, nx, ny, dx, dy);
    
    // declare the variable to store the x beam map
    double *cax;
    plhs[5] = mxCreateDoubleMatrix(1, ((int) nx[0])*((int) ny[0])*2, mxREAL);
    cax = mxGetPr(plhs[5]);
    
    // if it's a polarized beam, declare the variable to store the y beam map
    double *cay;
    if((int) is_polarized[0]) {
        plhs[6] = mxCreateDoubleMatrix(1, ((int) nx[0])*((int) ny[0])*2, mxREAL);
    } else {
        plhs[6] = mxCreateDoubleMatrix(1, 1, mxREAL);
    }
    cay = mxGetPr(plhs[6]);
    
    // get the beam maps
    get_beam_maps(out, cax, cay, is_polarized, nx, ny);
}